@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 # -*- Mode: Python; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+# vim: ft=python ts=4 sts=4 sw=4 et ai
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,60 +16,19 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# Copyright (C) 2014 Red Hat, Inc.
+# Copyright 2014 Red Hat, Inc.
 #
 
 #
 # This example updates a connection's IPv4 method with the Update() method
-# using the libnm-glib GObject-based convenience APIs.
+# using the libnm GObject-based convenience APIs.
 #
 # Configuration settings are described at
-# https://developer.gnome.org/NetworkManager/0.9/ref-settings.html
+# https://developer.gnome.org/NetworkManager/1.0/ref-settings.html
 #
 
-from gi.repository import GLib, NetworkManager, NMClient
-import sys, struct, socket
-
-def ip_to_int(ip_string):
-    return struct.unpack("=I", socket.inet_aton(ip_string))[0]
-
-# callback function
-def commit_cb(connection, error, data):
-    if error is (None):
-        print("The connection profile has been updated.")
-    else:
-        print(error)
-    main_loop.quit()
-
-def connections_read_cb(settings, data):
-    uuid, method, args = data
-
-    all_connections = settings.list_connections()
-    for c in all_connections:
-        if c.get_uuid() != uuid:
-            continue
-
-        # add IPv4 setting if it doesn't yet exist
-        s_ip4 = c.get_setting_ip4_config()
-        if not s_ip4:
-            s_ip4 = NetworkManager.SettingIP4Config.new()
-            c.add_setting(s_ip4)
-
-        # set the method and change properties
-        s_ip4.set_property(NetworkManager.SETTING_IP4_CONFIG_METHOD, method)
-        if method == "auto":
-            # remove addresses
-            s_ip4.clear_addresses()
-        elif method == "manual":
-            # Add the static IP address, prefix, and (optional) gateway
-            addr = NetworkManager.IP4Address.new()
-            addr.set_address(ip_to_int(sys.argv[3]))
-            addr.set_prefix(int(sys.argv[4]))
-            if len(sys.argv) == 6:
-                addr.set_gateway(ip_to_int(sys.argv[5]))
-            s_ip4.add_address(addr)
-
-        c.commit_changes(commit_cb, None)
+from gi.repository import GLib, NM
+import sys, socket
 
 if __name__ == "__main__":
     # parse and validate arguments
@@ -77,20 +37,49 @@ if __name__ == "__main__":
         sys.exit(1)
 
     method = sys.argv[2]
-    if method == "static" and len(sys.argv) < 5:
+    if (method == "static" or method == "manual") and len(sys.argv) < 5:
         print "Usage: %s %s static address prefix [gateway]" % (sys.argv[0], sys.argv[1])
         sys.exit(1)
 
+    uuid = sys.argv[1]
+
     # Convert method to NM method
     if method == "static":
         method = "manual"
 
     main_loop = GLib.MainLoop()
 
-    # create RemoteSettings object and attach to the "connections-read" signal
-    # to wait for connections to be loaded asynchronously
-    settings = NMClient.RemoteSettings.new(None)
-    settings.connect('connections-read', connections_read_cb, (sys.argv[1], method, sys.argv))
+    # create Client object
+    client = NM.Client.new(None)
 
-    main_loop.run()
+    all_connections = client.get_connections()
+    for c in all_connections:
+        if c.get_uuid() != uuid:
+            continue
+
+        # add IPv4 setting if it doesn't yet exist
+        s_ip4 = c.get_setting_ip4_config()
+        if not s_ip4:
+            s_ip4 = NM.SettingIP4Config.new()
+            c.add_setting(s_ip4)
+
+        # set the method and change properties
+        s_ip4.set_property(NM.SETTING_IP_CONFIG_METHOD, method)
+        if method == "auto":
+            # remove addresses and gateway
+            s_ip4.clear_addresses()
+            s_ip4.props.gateway = None
+        elif method == "manual":
+            # Add the static IP address, prefix, and (optional) gateway
+            addr = NM.IPAddress.new(socket.AF_INET, sys.argv[3], int(sys.argv[4]))
+            s_ip4.add_address(addr)
+            if len(sys.argv) == 6:
+                s_ip4.props.gateway = sys.argv[5]
+
+        try:
+            c.commit_changes(True, None)
+            print("The connection profile has been updated.")
+        except Exception, e:
+            print("Error: %s" % e)
+        break
 
