@@ -204,6 +204,9 @@ typedef enum {
  * @NM_WIFI_DEVICE_CAP_RSN: device supports WPA2/RSN authentication
  * @NM_WIFI_DEVICE_CAP_AP: device supports Access Point mode
  * @NM_WIFI_DEVICE_CAP_ADHOC: device supports Ad-Hoc mode
+ * @NM_WIFI_DEVICE_CAP_FREQ_VALID: device reports frequency capabilities
+ * @NM_WIFI_DEVICE_CAP_FREQ_2GHZ: device supports 2.4GHz frequencies
+ * @NM_WIFI_DEVICE_CAP_FREQ_5GHZ: device supports 5GHz frequencies
  *
  * 802.11 specific device encryption and authentication capabilities.
  *
@@ -218,7 +221,10 @@ typedef enum {
 	NM_WIFI_DEVICE_CAP_WPA           = 0x00000010,
 	NM_WIFI_DEVICE_CAP_RSN           = 0x00000020,
 	NM_WIFI_DEVICE_CAP_AP            = 0x00000040,
-	NM_WIFI_DEVICE_CAP_ADHOC         = 0x00000080
+	NM_WIFI_DEVICE_CAP_ADHOC         = 0x00000080,
+	NM_WIFI_DEVICE_CAP_FREQ_VALID    = 0x00000100,
+	NM_WIFI_DEVICE_CAP_FREQ_2GHZ     = 0x00000200,
+	NM_WIFI_DEVICE_CAP_FREQ_5GHZ     = 0x00000400,
 } NMDeviceWifiCapabilities;
 
 
@@ -465,6 +471,7 @@ typedef enum {
  * @NM_DEVICE_STATE_REASON_MODEM_FAILED: Modem failed or no longer available
  * @NM_DEVICE_STATE_REASON_MODEM_AVAILABLE: Modem now ready and available
  * @NM_DEVICE_STATE_REASON_SIM_PIN_INCORRECT: SIM PIN was incorrect
+ * @NM_DEVICE_STATE_REASON_NEW_ACTIVATION: New connection activation was enqueued
  *
  * Device state change reason codes
  *
@@ -531,6 +538,7 @@ typedef enum {
 	NM_DEVICE_STATE_REASON_MODEM_FAILED = 57,
 	NM_DEVICE_STATE_REASON_MODEM_AVAILABLE = 58,
 	NM_DEVICE_STATE_REASON_SIM_PIN_INCORRECT = 59,
+	NM_DEVICE_STATE_REASON_NEW_ACTIVATION = 60,
 
 	NM_DEVICE_STATE_REASON_LAST = 0xFFFF
 } NMDeviceStateReason;
