@@ -246,6 +246,12 @@ test_ip6_address_external (void)
 	free_signal (address_removed);
 }
 
+void
+init_tests (int *argc, char ***argv)
+{
+	nmtst_init_with_logging (argc, argv, NULL, "ALL");
+}
+
 void
 setup_tests (void)
 {
