@@ -86,6 +86,12 @@ test_cleanup_internal (void)
 	g_array_unref (routes6);
 }
 
+void
+init_tests (int *argc, char ***argv)
+{
+	nmtst_init_with_logging (argc, argv, NULL, "ALL");
+}
+
 void
 setup_tests (void)
 {
