@@ -58,10 +58,31 @@
 #include "reader.h"
 #include "writer.h"
 #include "utils.h"
+#include "gsystem-local-alloc.h"
 
 #define DBUS_SERVICE_NAME "com.redhat.ifcfgrh1"
 #define DBUS_OBJECT_PATH "/com/redhat/ifcfgrh1"
 
+
+#define _LOG_DEFAULT_DOMAIN  LOGD_SETTINGS
+
+#define _LOG(level, domain, ...) \
+    G_STMT_START { \
+        nm_log ((level), (domain), \
+                "%s" _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+                "ifcfg-rh: " \
+                _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+    } G_STMT_END
+
+#define _LOGT(...)      _LOG (LOGL_TRACE, _LOG_DEFAULT_DOMAIN, __VA_ARGS__)
+#define _LOGD(...)      _LOG (LOGL_DEBUG, _LOG_DEFAULT_DOMAIN, __VA_ARGS__)
+#define _LOGI(...)      _LOG (LOGL_INFO,  _LOG_DEFAULT_DOMAIN, __VA_ARGS__)
+#define _LOGW(...)      _LOG (LOGL_WARN,  _LOG_DEFAULT_DOMAIN, __VA_ARGS__)
+#define _LOGE(...)      _LOG (LOGL_ERR,   _LOG_DEFAULT_DOMAIN, __VA_ARGS__)
+
+#define ERR_GET_MSG(err) (((err) && (err)->message) ? (err)->message : "(unknown)")
+
+
 static gboolean impl_ifcfgrh_get_ifcfg_details (SCPluginIfcfg *plugin,
                                                 const char *in_ifcfg,
                                                 const char **out_uuid,
@@ -70,10 +91,13 @@ static gboolean impl_ifcfgrh_get_ifcfg_details (SCPluginIfcfg *plugin,
 
 #include "nm-ifcfg-rh-glue.h"
 
-static void connection_new_or_changed (SCPluginIfcfg *plugin,
-                                       const char *path,
-                                       NMIfcfgConnection *existing,
-                                       char **out_old_path);
+static NMIfcfgConnection *update_connection (SCPluginIfcfg *plugin,
+                                             NMConnection *source,
+                                             const char *full_path,
+                                             NMIfcfgConnection *connection,
+                                             gboolean protect_existing_connection,
+                                             GHashTable *protected_connections,
+                                             GError **error);
 
 static void system_config_interface_init (NMSystemConfigInterface *system_config_interface_class);
 
@@ -104,13 +128,22 @@ typedef struct {
 static void
 connection_ifcfg_changed (NMIfcfgConnection *connection, gpointer user_data)
 {
-	SCPluginIfcfg *plugin = SC_PLUGIN_IFCFG (user_data);
+	SCPluginIfcfg *self = SC_PLUGIN_IFCFG (user_data);
+	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (self);
 	const char *path;
 
-	path = nm_ifcfg_connection_get_path (connection);
+	path = nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection));
 	g_return_if_fail (path != NULL);
 
-	connection_new_or_changed (plugin, path, connection, NULL);
+
+	if (!priv->ifcfg_monitor) {
+		_LOGD ("connection_ifcfg_changed("NM_IFCFG_CONNECTION_LOG_FMTD"): %s", NM_IFCFG_CONNECTION_LOG_ARGD (connection), "ignore event");
+		return;
+	}
+
+	_LOGD ("connection_ifcfg_changed("NM_IFCFG_CONNECTION_LOG_FMTD"): %s", NM_IFCFG_CONNECTION_LOG_ARGD (connection), "reload");
+
+	update_connection (self, NULL, path, connection, TRUE, NULL, NULL);
 }
 
 static void
@@ -120,69 +153,6 @@ connection_removed_cb (NMSettingsConnection *obj, gpointer user_data)
 	                     nm_connection_get_uuid (NM_CONNECTION (obj)));
 }
 
-static NMIfcfgConnection *
-_internal_new_connection (SCPluginIfcfg *self,
-                          const char *path,
-                          NMConnection *source,
-                          GError **error)
-{
-	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (self);
-	NMIfcfgConnection *connection;
-	const char *cid;
-	GError *local = NULL;
-	gboolean ignore_error = FALSE;
-
-	if (!source)
-		nm_log_info (LOGD_SETTINGS, "parsing %s ... ", path);
-
-	connection = nm_ifcfg_connection_new (source, path, &local, &ignore_error);
-	if (!connection) {
-		if (!ignore_error)
-			nm_log_warn (LOGD_SETTINGS, "    %s", (local && local->message) ? local->message : "(unknown)");
-		if (local)
-			g_propagate_error (error, local);
-		else
-			g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INVALID_CONNECTION,
-			             "(unknown)");
-		return NULL;
-	}
-
-	cid = nm_connection_get_id (NM_CONNECTION (connection));
-	g_assert (cid);
-
-	g_hash_table_insert (priv->connections,
-	                     g_strdup (nm_connection_get_uuid (NM_CONNECTION (connection))),
-	                     connection);
-	nm_log_info (LOGD_SETTINGS, "    read connection '%s'", cid);
-	g_signal_connect (connection, NM_SETTINGS_CONNECTION_REMOVED,
-	                  G_CALLBACK (connection_removed_cb),
-	                  self);
-
-	if (nm_ifcfg_connection_get_unmanaged_spec (connection)) {
-		const char *spec;
-		const char *device_id;
-
-		spec = nm_ifcfg_connection_get_unmanaged_spec (connection);
-		device_id = strchr (spec, ':');
-		if (device_id)
-			device_id++;
-		else
-			device_id = spec;
-		nm_log_warn (LOGD_SETTINGS, "    Ignoring connection '%s' / device '%s' due to NM_CONTROLLED=no.",
-		             cid, device_id);
-	} else if (nm_ifcfg_connection_get_unrecognized_spec (connection)) {
-		nm_log_warn (LOGD_SETTINGS, "    Ignoring connection '%s' of unrecognized type.", cid);
-	}
-
-	/* watch changes of ifcfg hardlinks */
-	g_signal_connect (G_OBJECT (connection), "ifcfg-changed",
-	                  G_CALLBACK (connection_ifcfg_changed), self);
-
-	return connection;
-}
-
-/* Monitoring */
-
 static void
 remove_connection (SCPluginIfcfg *self, NMIfcfgConnection *connection)
 {
@@ -192,6 +162,8 @@ remove_connection (SCPluginIfcfg *self, NMIfcfgConnection *connection)
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (connection != NULL);
 
+	_LOGI ("remove "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection));
+
 	unmanaged = !!nm_ifcfg_connection_get_unmanaged_spec (connection);
 	unrecognized = !!nm_ifcfg_connection_get_unrecognized_spec (connection);
 
@@ -212,160 +184,233 @@ find_by_path (SCPluginIfcfg *self, const char *path)
 {
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (self);
 	GHashTableIter iter;
-	NMIfcfgConnection *candidate = NULL;
+	NMSettingsConnection *candidate = NULL;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
 	g_hash_table_iter_init (&iter, priv->connections);
 	while (g_hash_table_iter_next (&iter, NULL, (gpointer) &candidate)) {
-		if (g_strcmp0 (path, nm_ifcfg_connection_get_path (candidate)) == 0)
-			return candidate;
+		if (g_strcmp0 (path, nm_settings_connection_get_filename (candidate)) == 0)
+			return NM_IFCFG_CONNECTION (candidate);
 	}
 	return NULL;
 }
 
 static NMIfcfgConnection *
-find_by_uuid_from_path (SCPluginIfcfg *self, const char *path)
+update_connection (SCPluginIfcfg *self,
+                   NMConnection *source,
+                   const char *full_path,
+                   NMIfcfgConnection *connection,
+                   gboolean protect_existing_connection,
+                   GHashTable *protected_connections,
+                   GError **error)
 {
 	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (self);
-	char *uuid;
-
-	g_return_val_if_fail (path != NULL, NULL);
-
-	uuid = uuid_from_file (path);
-	if (uuid)
-		return g_hash_table_lookup (priv->connections, uuid);
-	else
-		return NULL;
-}
-
-static void
-connection_new_or_changed (SCPluginIfcfg *self,
-                           const char *path,
-                           NMIfcfgConnection *existing,
-                           char **out_old_path)
-{
-	SCPluginIfcfgPrivate *priv = SC_PLUGIN_IFCFG_GET_PRIVATE (self);
-	NMIfcfgConnection *new;
-	GError *error = NULL;
-	gboolean ignore_error = FALSE;
+	NMIfcfgConnection *connection_new;
+	NMIfcfgConnection *connection_by_uuid;
+	GError *local = NULL;
 	const char *new_unmanaged = NULL, *old_unmanaged = NULL;
 	const char *new_unrecognized = NULL, *old_unrecognized = NULL;
-	gboolean unmanaged_changed, unrecognized_changed;
+	gboolean unmanaged_changed = FALSE, unrecognized_changed = FALSE;
+	const char *uuid;
 
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (path != NULL);
+	g_return_val_if_fail (!source || NM_IS_CONNECTION (source), NULL);
+	g_return_val_if_fail (full_path || source, NULL);
 
-	if (out_old_path)
-		*out_old_path = NULL;
+	if (full_path)
+		_LOGD ("loading from file \"%s\"...", full_path);
 
-	if (!existing) {
-		/* See if it's a rename */
-		existing = find_by_uuid_from_path (self, path);
-		if (existing) {
-			const char *old_path = nm_ifcfg_connection_get_path (existing);
-			nm_log_info (LOGD_SETTINGS, "renaming %s -> %s", old_path, path);
-			if (out_old_path)
-				*out_old_path = g_strdup (old_path);
-			nm_ifcfg_connection_set_path (existing, path);
-		}
+	/* Create a NMIfcfgConnection instance, either by reading from @full_path or
+	 * based on @source. */
+	connection_new = nm_ifcfg_connection_new (source, full_path, error);
+	if (!connection_new) {
+		/* Unexpected failure. Probably the file is invalid? */
+		if (   connection
+		    && !protect_existing_connection
+		    && (!protected_connections || !g_hash_table_contains (protected_connections, connection)))
+			remove_connection (self, connection);
+		return NULL;
 	}
 
-	if (!existing) {
-		/* New connection */
-		new = _internal_new_connection (self, path, NULL, NULL);
-		if (new) {
-			if (nm_ifcfg_connection_get_unmanaged_spec (new))
-				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
-			else if (nm_ifcfg_connection_get_unrecognized_spec (new))
-				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNRECOGNIZED_SPECS_CHANGED);
+	uuid = nm_connection_get_uuid (NM_CONNECTION (connection_new));
+	connection_by_uuid = g_hash_table_lookup (priv->connections, uuid);
+
+	if (   connection
+	    && connection != connection_by_uuid) {
+
+		if (   (protect_existing_connection && connection_by_uuid != NULL)
+		    || (protected_connections && g_hash_table_contains (protected_connections, connection))) {
+			NMIfcfgConnection *conflicting = (protect_existing_connection && connection_by_uuid != NULL) ? connection_by_uuid : connection;
+
+			if (source)
+				_LOGW ("cannot update protected connection "NM_IFCFG_CONNECTION_LOG_FMT" due to conflicting UUID %s", NM_IFCFG_CONNECTION_LOG_ARG (conflicting), uuid);
 			else
-				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, new);
+				_LOGW ("cannot load %s due to conflicting UUID for "NM_IFCFG_CONNECTION_LOG_FMT, full_path, NM_IFCFG_CONNECTION_LOG_ARG (conflicting));
+			g_object_unref (connection_new);
+			g_set_error_literal (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_FAILED,
+			                     "Cannot update protected connection due to conflicting UUID");
+			return NULL;
 		}
-		return;
+
+		/* The new connection has a different UUID then the original one that we
+		 * are about to update. Remove @connection. */
+		remove_connection (self, connection);
 	}
 
-	new = (NMIfcfgConnection *) nm_ifcfg_connection_new (NULL, path, &error, &ignore_error);
-	if (!new) {
-		/* errors reading connection; remove it */
-		if (!ignore_error)
-			nm_log_warn (LOGD_SETTINGS, "    %s", (error && error->message) ? error->message : "(unknown)");
-		g_clear_error (&error);
-
-		nm_log_info (LOGD_SETTINGS, "removed %s.", path);
-		remove_connection (self, existing);
-		return;
+	/* Check if the found connection with the same UUID is not protected from updating. */
+	if (   connection_by_uuid
+	    && (   (!connection && protect_existing_connection)
+	        || (protected_connections && g_hash_table_contains (protected_connections, connection_by_uuid)))) {
+		if (source)
+			_LOGW ("cannot update connection due to conflicting UUID for "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection_by_uuid));
+		else
+			_LOGW ("cannot load %s due to conflicting UUID for "NM_IFCFG_CONNECTION_LOG_FMT, full_path, NM_IFCFG_CONNECTION_LOG_ARG (connection_by_uuid));
+		g_object_unref (connection_new);
+		g_set_error_literal (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_FAILED,
+		                      "Skip updating protected connection during reload");
+		return NULL;
 	}
 
-	/* Successfully read connection changes */
-
-	old_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (NM_IFCFG_CONNECTION (existing));
-	new_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (NM_IFCFG_CONNECTION (new));
+	/* Evaluate unmanaged/unrecognized flags. */
+	if (connection_by_uuid)
+		old_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (connection_by_uuid);
+	new_unmanaged = nm_ifcfg_connection_get_unmanaged_spec (connection_new);
 	unmanaged_changed = g_strcmp0 (old_unmanaged, new_unmanaged);
 
-	old_unrecognized = nm_ifcfg_connection_get_unrecognized_spec (NM_IFCFG_CONNECTION (existing));
-	new_unrecognized = nm_ifcfg_connection_get_unrecognized_spec (NM_IFCFG_CONNECTION (new));
+	if (connection_by_uuid)
+		old_unrecognized = nm_ifcfg_connection_get_unrecognized_spec (connection_by_uuid);
+	new_unrecognized = nm_ifcfg_connection_get_unrecognized_spec (connection_new);
 	unrecognized_changed = g_strcmp0 (old_unrecognized, new_unrecognized);
 
-	if (   !unmanaged_changed
-	    && !unrecognized_changed
-	    && nm_connection_compare (NM_CONNECTION (existing),
-	                              NM_CONNECTION (new),
-	                              NM_SETTING_COMPARE_FLAG_IGNORE_AGENT_OWNED_SECRETS |
-	                                  NM_SETTING_COMPARE_FLAG_IGNORE_NOT_SAVED_SECRETS)) {
-		g_object_unref (new);
-		return;
-	}
+	if (connection_by_uuid) {
+		const char *old_path;
 
-	nm_log_info (LOGD_SETTINGS, "updating %s", path);
-	g_object_set (existing,
-	              NM_IFCFG_CONNECTION_UNMANAGED_SPEC, new_unmanaged,
-	              NM_IFCFG_CONNECTION_UNRECOGNIZED_SPEC, new_unrecognized,
-	              NULL);
+		old_path = nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection_by_uuid));
 
-	if (new_unmanaged || new_unrecognized) {
-		if (!old_unmanaged && !old_unrecognized) {
-			g_object_ref (existing);
-			/* Unexport the connection by telling the settings service it's
-			 * been removed.
-			 */
-			nm_settings_connection_signal_remove (NM_SETTINGS_CONNECTION (existing));
-			/* Remove the path so that claim_connection() doesn't complain later when
-			 * interface gets managed and connection is re-added. */
-			nm_connection_set_path (NM_CONNECTION (existing), NULL);
+		if (   !unmanaged_changed
+		    && !unrecognized_changed
+		    && nm_connection_compare (NM_CONNECTION (connection_by_uuid),
+		                              NM_CONNECTION (connection_new),
+		                              NM_SETTING_COMPARE_FLAG_IGNORE_AGENT_OWNED_SECRETS |
+		                              NM_SETTING_COMPARE_FLAG_IGNORE_NOT_SAVED_SECRETS)) {
+			if (old_path && g_strcmp0 (old_path, full_path) != 0)
+				_LOGI ("rename \"%s\" to "NM_IFCFG_CONNECTION_LOG_FMT" without other changes", nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection_by_uuid)), NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+		} else {
 
-			/* signal_remove() will end up removing the connection from our hash,
-			 * so add it back now.
-			 */
-			g_hash_table_insert (priv->connections,
-			                     g_strdup (nm_connection_get_uuid (NM_CONNECTION (existing))),
-			                     existing);
+			/*******************************************************
+			 * UPDATE
+			 *******************************************************/
+
+			if (source)
+				_LOGI ("update "NM_IFCFG_CONNECTION_LOG_FMT" from %s", NM_IFCFG_CONNECTION_LOG_ARG (connection_new), NM_IFCFG_CONNECTION_LOG_PATH (old_path));
+			else if (!g_strcmp0 (old_path, nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection_new))))
+				_LOGI ("update "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+			else if (old_path)
+				_LOGI ("rename \"%s\" to "NM_IFCFG_CONNECTION_LOG_FMT, old_path, NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+			else
+				_LOGI ("update and persist "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+
+			g_object_set (connection_by_uuid,
+			              NM_IFCFG_CONNECTION_UNMANAGED_SPEC, new_unmanaged,
+			              NM_IFCFG_CONNECTION_UNRECOGNIZED_SPEC, new_unrecognized,
+			              NULL);
+
+			if (!nm_settings_connection_replace_settings (NM_SETTINGS_CONNECTION (connection_by_uuid),
+			                                              NM_CONNECTION (connection_new),
+			                                              FALSE,  /* don't set Unsaved */
+			                                              "ifcfg-update",
+			                                              &local)) {
+				/* Shouldn't ever get here as 'connection_new' was verified by the reader already
+				 * and the UUID did not change. */
+				g_assert_not_reached ();
+			}
+			g_assert_no_error (local);
+
+			if (new_unmanaged || new_unrecognized) {
+				if (!old_unmanaged && !old_unrecognized) {
+					g_object_ref (connection_by_uuid);
+					/* Unexport the connection by telling the settings service it's
+					 * been removed.
+					 */
+					nm_settings_connection_signal_remove (NM_SETTINGS_CONNECTION (connection_by_uuid));
+					/* Remove the path so that claim_connection() doesn't complain later when
+					 * interface gets managed and connection is re-added. */
+					nm_connection_set_path (NM_CONNECTION (connection_by_uuid), NULL);
+
+					/* signal_remove() will end up removing the connection from our hash,
+					 * so add it back now.
+					 */
+					g_hash_table_insert (priv->connections,
+					                     g_strdup (nm_connection_get_uuid (NM_CONNECTION (connection_by_uuid))),
+					                     connection_by_uuid);
+				}
+			} else {
+				if (old_unmanaged /* && !new_unmanaged */) {
+					_LOGI ("Managing connection "NM_IFCFG_CONNECTION_LOG_FMT" and its device because NM_CONTROLLED was true.",
+					       NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+					g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, connection_by_uuid);
+				} else if (old_unrecognized /* && !new_unrecognized */) {
+					_LOGI ("Managing connection "NM_IFCFG_CONNECTION_LOG_FMT" because it is now a recognized type.",
+					       NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+					g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, connection_by_uuid);
+				}
+			}
+
+			if (unmanaged_changed)
+				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
+			if (unrecognized_changed)
+				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNRECOGNIZED_SPECS_CHANGED);
 		}
+		nm_settings_connection_set_filename (NM_SETTINGS_CONNECTION (connection_by_uuid), full_path);
+		g_object_unref (connection_new);
+		return connection_by_uuid;
 	} else {
-		const char *cid = nm_connection_get_id (NM_CONNECTION (new));
 
-		if (old_unmanaged /* && !new_unmanaged */) {
-			nm_log_info (LOGD_SETTINGS, "Managing connection '%s' and its device because NM_CONTROLLED was true.", cid);
-			g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, existing);
-		} else if (old_unrecognized /* && !new_unrecognized */) {
-			nm_log_info (LOGD_SETTINGS, "Managing connection '%s' because it is now a recognized type.", cid);
-			g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, existing);
-		}
+		/*******************************************************
+		 * ADD
+		 *******************************************************/
 
-		if (!nm_settings_connection_replace_settings (NM_SETTINGS_CONNECTION (existing),
-		                                              NM_CONNECTION (new),
-		                                              FALSE,  /* don't set Unsaved */
-		                                              &error)) {
-			/* Shouldn't ever get here as 'new' was verified by the reader already */
-			g_assert_no_error (error);
+		if (source)
+			_LOGI ("add connection "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+		else
+			_LOGI ("new connection "NM_IFCFG_CONNECTION_LOG_FMT, NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+		g_hash_table_insert (priv->connections, g_strdup (uuid), connection_new);
+
+		g_signal_connect (connection_new, NM_SETTINGS_CONNECTION_REMOVED,
+		                  G_CALLBACK (connection_removed_cb),
+		                  self);
+
+		if (nm_ifcfg_connection_get_unmanaged_spec (connection_new)) {
+			const char *spec;
+			const char *device_id;
+
+			spec = nm_ifcfg_connection_get_unmanaged_spec (connection_new);
+			device_id = strchr (spec, ':');
+			if (device_id)
+				device_id++;
+			else
+				device_id = spec;
+			_LOGW ("Ignoring connection "NM_IFCFG_CONNECTION_LOG_FMT" / device '%s' due to NM_CONTROLLED=no.",
+			       NM_IFCFG_CONNECTION_LOG_ARG (connection_new), device_id);
+		} else if (nm_ifcfg_connection_get_unrecognized_spec (connection_new))
+			_LOGW ("Ignoring connection "NM_IFCFG_CONNECTION_LOG_FMT" of unrecognized type.", NM_IFCFG_CONNECTION_LOG_ARG (connection_new));
+
+		/* watch changes of ifcfg hardlinks */
+		g_signal_connect (G_OBJECT (connection_new), "ifcfg-changed",
+		                  G_CALLBACK (connection_ifcfg_changed), self);
+
+		if (!source) {
+			/* Only raise the signal if we were called without source, i.e. if we read the connection from file.
+			 * Otherwise, we were called by add_connection() which does not expect the signal. */
+			if (nm_ifcfg_connection_get_unmanaged_spec (connection_new))
+				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
+			else if (nm_ifcfg_connection_get_unrecognized_spec (connection_new))
+				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNRECOGNIZED_SPECS_CHANGED);
+			else
+				g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_CONNECTION_ADDED, connection_new);
 		}
+		return connection_new;
 	}
-	g_object_unref (new);
-
-	if (unmanaged_changed)
-		g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNMANAGED_SPECS_CHANGED);
-	if (unrecognized_changed)
-		g_signal_emit_by_name (self, NM_SYSTEM_CONFIG_INTERFACE_UNRECOGNIZED_SPECS_CHANGED);
 }
 
 static void
@@ -385,6 +430,8 @@ ifcfg_dir_changed (GFileMonitor *monitor,
 		return;
 	}
 
+	_LOGD ("ifcfg_dir_changed(%s) = %d", path, event_type);
+
 	base = g_file_get_basename (file);
 	if (utils_is_ifcfg_alias_file (base, NULL)) {
 		/* Alias file changed. Get the base ifcfg file from it */
@@ -397,14 +444,13 @@ ifcfg_dir_changed (GFileMonitor *monitor,
 		connection = find_by_path (plugin, ifcfg_path);
 		switch (event_type) {
 		case G_FILE_MONITOR_EVENT_DELETED:
-			nm_log_info (LOGD_SETTINGS, "removed %s.", ifcfg_path);
 			if (connection)
 				remove_connection (plugin, connection);
 			break;
 		case G_FILE_MONITOR_EVENT_CREATED:
 		case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
 			/* Update or new */
-			connection_new_or_changed (plugin, ifcfg_path, connection, NULL);
+			update_connection (plugin, NULL, ifcfg_path, connection, TRUE, NULL, NULL);
 			break;
 		default:
 			break;
@@ -433,6 +479,43 @@ setup_ifcfg_monitoring (SCPluginIfcfg *plugin)
 	}
 }
 
+static GHashTable *
+_paths_from_connections (GHashTable *connections)
+{
+	GHashTableIter iter;
+	NMIfcfgConnection *connection;
+	GHashTable *paths = g_hash_table_new (g_str_hash, g_str_equal);
+
+	g_hash_table_iter_init (&iter, connections);
+	while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &connection)) {
+		const char *path = nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection));
+
+		if (path)
+			g_hash_table_add (paths, (void *) path);
+	}
+	return paths;
+}
+
+static int
+_sort_paths (const char **f1, const char **f2, GHashTable *paths)
+{
+	struct stat st;
+	gboolean c1, c2;
+	gint64 m1, m2;
+
+	c1 = !!g_hash_table_contains (paths, *f1);
+	c2 = !!g_hash_table_contains (paths, *f2);
+	if (c1 != c2)
+		return c1 ? -1 : 1;
+
+	m1 = stat (*f1, &st) == 0 ? (gint64) st.st_mtime : G_MININT64;
+	m2 = stat (*f2, &st) == 0 ? (gint64) st.st_mtime : G_MININT64;
+	if (m1 != m2)
+		return m1 > m2 ? -1 : 1;
+
+	return strcmp (*f1, *f2);
+}
+
 static void
 read_connections (SCPluginIfcfg *plugin)
 {
@@ -440,28 +523,26 @@ read_connections (SCPluginIfcfg *plugin)
 	GDir *dir;
 	GError *err = NULL;
 	const char *item;
-	GHashTable *oldconns;
+	GHashTable *alive_connections;
 	GHashTableIter iter;
-	gpointer key, value;
 	NMIfcfgConnection *connection;
+	GPtrArray *dead_connections = NULL;
+	guint i;
+	GPtrArray *filenames;
+	GHashTable *paths;
 
 	dir = g_dir_open (IFCFG_DIR, 0, &err);
 	if (!dir) {
-		nm_log_warn (LOGD_SETTINGS, "Could not read directory '%s': %s", IFCFG_DIR, err->message);
+		_LOGW ("Could not read directory '%s': %s", IFCFG_DIR, err->message);
 		g_error_free (err);
 		return;
 	}
 
-	oldconns = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	g_hash_table_iter_init (&iter, priv->connections);
-	while (g_hash_table_iter_next (&iter, NULL, &value)) {
-		const char *ifcfg_path = nm_ifcfg_connection_get_path (value);
-		if (ifcfg_path)
-			g_hash_table_insert (oldconns, g_strdup (ifcfg_path), value);
-	}
+	alive_connections = g_hash_table_new (NULL, NULL);
 
+	filenames = g_ptr_array_new_with_free_func (g_free);
 	while ((item = g_dir_read_name (dir))) {
-		char *full_path, *old_path;
+		char *full_path;
 
 		if (utils_should_ignore_file (item, TRUE))
 			continue;
@@ -470,31 +551,45 @@ read_connections (SCPluginIfcfg *plugin)
 
 		full_path = g_build_filename (IFCFG_DIR, item, NULL);
 		if (!utils_get_ifcfg_name (full_path, TRUE))
-			goto next;
-
-		connection = g_hash_table_lookup (oldconns, full_path);
-		g_hash_table_remove (oldconns, full_path);
-		connection_new_or_changed (plugin, full_path, connection, &old_path);
-
-		if (old_path) {
-			g_hash_table_remove (oldconns, old_path);
-			g_free (old_path);
-		}
-
-	next:
-		g_free (full_path);
+			g_free (full_path);
+		else
+			g_ptr_array_add (filenames, full_path);
 	}
-
 	g_dir_close (dir);
 
-	g_hash_table_iter_init (&iter, oldconns);
-	while (g_hash_table_iter_next (&iter, &key, &value)) {
-		nm_log_info (LOGD_SETTINGS, "removed %s.", (char *)key);
-		g_hash_table_iter_remove (&iter);
-		remove_connection (plugin, value);
-	}
+	/* While reloading, we don't replace connections that we already loaded while
+	 * iterating over the files.
+	 *
+	 * To have sensible, reproducible behavior, sort the paths by last modification
+	 * time prefering older files.
+	 */
+	paths = _paths_from_connections (priv->connections);
+	g_ptr_array_sort_with_data (filenames, (GCompareDataFunc) _sort_paths, paths);
+	g_hash_table_destroy (paths);
 
-	g_hash_table_destroy (oldconns);
+	for (i = 0; i < filenames->len; i++) {
+		connection = update_connection (plugin, NULL, filenames->pdata[i], NULL, FALSE, alive_connections, NULL);
+		if (connection)
+			g_hash_table_add (alive_connections, connection);
+	}
+	g_ptr_array_free (filenames, TRUE);
+
+	g_hash_table_iter_init (&iter, priv->connections);
+	while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &connection)) {
+		if (   !g_hash_table_contains (alive_connections, connection)
+		    && nm_settings_connection_get_filename (NM_SETTINGS_CONNECTION (connection))) {
+			if (!dead_connections)
+				dead_connections = g_ptr_array_new ();
+			g_ptr_array_add (dead_connections, connection);
+		}
+	}
+	g_hash_table_destroy (alive_connections);
+
+	if (dead_connections) {
+		for (i = 0; i < dead_connections->len; i++)
+			remove_connection (plugin, dead_connections->pdata[i]);
+		g_ptr_array_free (dead_connections, TRUE);
+	}
 }
 
 static GSList *
@@ -540,7 +635,7 @@ load_connection (NMSystemConfigInterface *config,
 		return FALSE;
 
 	connection = find_by_path (plugin, filename);
-	connection_new_or_changed (plugin, filename, connection, NULL);
+	update_connection (plugin, NULL, filename, connection, TRUE, NULL, NULL);
 	if (!connection)
 		connection = find_by_path (plugin, filename);
 
@@ -605,8 +700,7 @@ add_connection (NMSystemConfigInterface *config,
                 GError **error)
 {
 	SCPluginIfcfg *self = SC_PLUGIN_IFCFG (config);
-	NMIfcfgConnection *added = NULL;
-	char *path = NULL;
+	gs_free char *path = NULL;
 
 	/* Ensure we reject attempts to add the connection long before we're
 	 * asked to write it to disk.
@@ -618,10 +712,7 @@ add_connection (NMSystemConfigInterface *config,
 		if (!writer_new_connection (connection, IFCFG_DIR, &path, error))
 			return NULL;
 	}
-
-	added = _internal_new_connection (self, path, connection, error);
-	g_free (path);
-	return (NMSettingsConnection *) added;
+	return NM_SETTINGS_CONNECTION (update_connection (self, connection, path, NULL, FALSE, NULL, error));
 }
 
 #define SC_NETWORK_FILE "/etc/sysconfig/network"
@@ -641,7 +732,7 @@ plugin_get_hostname (SCPluginIfcfg *plugin)
 
 	network = svOpenFile (SC_NETWORK_FILE, NULL);
 	if (!network) {
-		nm_log_warn (LOGD_SETTINGS, "Could not get hostname: failed to read " SC_NETWORK_FILE);
+		_LOGW ("Could not get hostname: failed to read " SC_NETWORK_FILE);
 		return NULL;
 	}
 
@@ -693,7 +784,7 @@ plugin_set_hostname (SCPluginIfcfg *plugin, const char *hostname)
 #endif
 
 	if (!ret) {
-		nm_log_warn (LOGD_SETTINGS, "Could not save hostname: failed to create/open " HOSTNAME_FILE);
+		_LOGW ("Could not save hostname: failed to create/open " HOSTNAME_FILE);
 		g_free (hostname_eol);
 		return FALSE;
 	}
@@ -862,7 +953,7 @@ sc_plugin_ifcfg_init (SCPluginIfcfg *plugin)
 
 	priv->bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
 	if (!priv->bus) {
-		nm_log_warn (LOGD_SETTINGS, "Couldn't connect to D-Bus: %s", error->message);
+		_LOGW ("Couldn't connect to D-Bus: %s", error->message);
 		g_clear_error (&error);
 	} else {
 		DBusConnection *tmp;
@@ -883,10 +974,10 @@ sc_plugin_ifcfg_init (SCPluginIfcfg *plugin)
 		                        G_TYPE_INVALID,
 		                        G_TYPE_UINT, &result,
 		                        G_TYPE_INVALID)) {
-			nm_log_warn (LOGD_SETTINGS, "Couldn't acquire D-Bus service: %s", error->message);
+			_LOGW ("Couldn't acquire D-Bus service: %s", error->message);
 			g_clear_error (&error);
 		} else if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-			nm_log_warn (LOGD_SETTINGS, "Couldn't acquire ifcfgrh1 D-Bus service (already taken)");
+			_LOGW ("Couldn't acquire ifcfgrh1 D-Bus service (already taken)");
 		} else
 			success = TRUE;
 	}
@@ -1048,7 +1139,7 @@ nm_system_config_factory (void)
 			dbus_g_connection_register_g_object (priv->bus,
 			                                     DBUS_OBJECT_PATH,
 			                                     G_OBJECT (singleton));
-		nm_log_info (LOGD_SETTINGS, "Acquired D-Bus service %s", DBUS_SERVICE_NAME);
+		_LOGI ("Acquired D-Bus service %s", DBUS_SERVICE_NAME);
 	} else
 		g_object_ref (singleton);
 
