@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <errno.h>
 
-#include <dbus/dbus-glib.h>
 #include <nm-setting.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
@@ -42,7 +41,6 @@
 #include <string.h>
 #include <arpa/inet.h>
 
-#include "nm-dbus-glib-types.h"
 #include "nm-glib-compat.h"
 #include "nm-logging.h"
 #include "writer.h"
@@ -345,19 +343,24 @@ password_raw_writer (GKeyFile *file,
                      const GValue *value)
 {
 	const char *setting_name = nm_setting_get_name (setting);
-	GByteArray *array;
-	int i, *tmp_array;
+	GBytes *array;
+	int *tmp_array;
+	gsize i, len;
+	const char *data;
 
-	g_return_if_fail (G_VALUE_HOLDS (value, DBUS_TYPE_G_UCHAR_ARRAY));
+	g_return_if_fail (G_VALUE_HOLDS (value, G_TYPE_BYTES));
 
-	array = (GByteArray *) g_value_get_boxed (value);
-	if (!array || !array->len)
+	array = (GBytes *) g_value_get_boxed (value);
+	if (!array)
+		return;
+	data = g_bytes_get_data (array, &len);
+	if (!data || !len)
 		return;
 
-	tmp_array = g_new (gint, array->len);
-	for (i = 0; i < array->len; i++)
-		tmp_array[i] = (int) array->data[i];
-	nm_keyfile_plugin_kf_set_integer_list (file, setting_name, key, tmp_array, array->len);
+	tmp_array = g_new (gint, len);
+	for (i = 0; i < len; i++)
+		tmp_array[i] = (int) data[i];
+	nm_keyfile_plugin_kf_set_integer_list (file, setting_name, key, tmp_array, len);
 	g_free (tmp_array);
 }
 
@@ -813,13 +816,10 @@ _internal_write_connection (NMConnection *connection,
 	WriteInfo info;
 	GError *local_err = NULL;
 
-	if (out_path)
-		g_return_val_if_fail (*out_path == NULL, FALSE);
+	g_return_val_if_fail (!out_path || !*out_path, FALSE);
 
-	if (!nm_connection_verify (connection, error)) {
+	if (!nm_connection_verify (connection, error))
 		g_return_val_if_reached (FALSE);
-		return FALSE;
-	}
 
 	id = nm_connection_get_id (connection);
 	g_assert (id && *id);
