@@ -19,6 +19,8 @@
  * Copyright (C) 2007 - 2008 Novell, Inc.
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <glib.h>
 #include <dbus/dbus.h>
@@ -405,11 +407,11 @@ nm_supplicant_manager_class_init (NMSupplicantManagerClass *klass)
 	object_class->set_property = set_property;
 	object_class->dispose = dispose;
 
-	g_object_class_install_property (object_class, PROP_AVAILABLE,
-		g_param_spec_boolean (NM_SUPPLICANT_MANAGER_AVAILABLE,
-		                      "Available",
-		                      "Available",
-		                      FALSE,
-		                      G_PARAM_READABLE));
+	g_object_class_install_property
+		(object_class, PROP_AVAILABLE,
+		 g_param_spec_boolean (NM_SUPPLICANT_MANAGER_AVAILABLE, "", "",
+		                       FALSE,
+		                       G_PARAM_READABLE |
+		                       G_PARAM_STATIC_STRINGS));
 }
 
