@@ -331,6 +331,33 @@ test_nm_utils_kill_child (void)
 }
 
 
+/*******************************************/
+
+static void
+test_nm_ethernet_address_is_valid ()
+{
+	g_assert (!nm_ethernet_address_is_valid (NULL, -1));
+	g_assert (!nm_ethernet_address_is_valid (NULL, ETH_ALEN));
+
+	g_assert (!nm_ethernet_address_is_valid ("FF:FF:FF:FF:FF:FF", -1));
+	g_assert (!nm_ethernet_address_is_valid ("00:00:00:00:00:00", -1));
+	g_assert (!nm_ethernet_address_is_valid ("44:44:44:44:44:44", -1));
+	g_assert (!nm_ethernet_address_is_valid ("00:30:b4:00:00:00", -1));
+
+	g_assert (!nm_ethernet_address_is_valid ("", -1));
+	g_assert (!nm_ethernet_address_is_valid ("1", -1));
+	g_assert (!nm_ethernet_address_is_valid ("2", -1));
+
+	g_assert (!nm_ethernet_address_is_valid (((guint8[8]) { 0x00,0x30,0xb4,0x00,0x00,0x00 }), ETH_ALEN));
+	g_assert ( nm_ethernet_address_is_valid (((guint8[8]) { 0x00,0x30,0xb4,0x00,0x00,0x01 }), ETH_ALEN));
+
+	/* some Broad cast addresses (with MSB of first octet set). */
+	g_assert (!nm_ethernet_address_is_valid ("57:44:44:44:44:44", -1));
+	g_assert ( nm_ethernet_address_is_valid ("56:44:44:44:44:44", -1));
+	g_assert (!nm_ethernet_address_is_valid (((guint8[8]) { 0x03,0x30,0xb4,0x00,0x00,0x00 }), ETH_ALEN));
+	g_assert ( nm_ethernet_address_is_valid (((guint8[8]) { 0x02,0x30,0xb4,0x00,0x00,0x01 }), ETH_ALEN));
+}
+
 /*******************************************/
 
 NMTST_DEFINE ();
@@ -338,11 +365,10 @@ NMTST_DEFINE ();
 int
 main (int argc, char **argv)
 {
-	nmtst_init_assert_logging (&argc, &argv);
-
-	nm_logging_setup ("DEBUG", "DEFAULT", NULL, NULL);
+	nmtst_init_assert_logging (&argc, &argv, "DEBUG", "DEFAULT");
 
 	g_test_add_func ("/general/nm_utils_kill_child", test_nm_utils_kill_child);
+	g_test_add_func ("/general/nm_ethernet_address_is_valid", test_nm_ethernet_address_is_valid);
 
 	return g_test_run ();
 }
