@@ -19,8 +19,8 @@
  * Copyright (C) 2005 - 2008 Novell, Inc.
  */
 
-#ifndef NM_VPN_SERVICE_H
-#define NM_VPN_SERVICE_H
+#ifndef __NETWORKMANAGER_VPN_SERVICE_H__
+#define __NETWORKMANAGER_VPN_SERVICE_H__
 
 #include <glib.h>
 #include <glib-object.h>
@@ -28,36 +28,36 @@
 #include "nm-vpn-connection.h"
 
 #define NM_TYPE_VPN_SERVICE            (nm_vpn_service_get_type ())
-#define NM_VPN_SERVICE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_VPN_SERVICE, NMVPNService))
-#define NM_VPN_SERVICE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_VPN_SERVICE, NMVPNServiceClass))
+#define NM_VPN_SERVICE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_VPN_SERVICE, NMVpnService))
+#define NM_VPN_SERVICE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_VPN_SERVICE, NMVpnServiceClass))
 #define NM_IS_VPN_SERVICE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NM_TYPE_VPN_SERVICE))
 #define NM_IS_VPN_SERVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_VPN_SERVICE))
-#define NM_VPN_SERVICE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_VPN_SERVICE, NMVPNServiceClass))
+#define NM_VPN_SERVICE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_VPN_SERVICE, NMVpnServiceClass))
 
 typedef struct {
 	GObject parent;
-} NMVPNService;
+} NMVpnService;
 
 typedef struct {
 	GObjectClass parent;
-} NMVPNServiceClass;
+} NMVpnServiceClass;
 
 GType nm_vpn_service_get_type (void);
 
-NMVPNService * nm_vpn_service_new (const char *namefile, GError **error);
+NMVpnService * nm_vpn_service_new (const char *namefile, GError **error);
 
 /* Returns the VPN service's D-Bus service name */
-const char *nm_vpn_service_get_dbus_service (NMVPNService *service);
+const char *nm_vpn_service_get_dbus_service (NMVpnService *service);
 
 /* Returns the path of the VPN service's .name file */
-const char *nm_vpn_service_get_name_file (NMVPNService *service);
+const char *nm_vpn_service_get_name_file (NMVpnService *service);
 
-gboolean nm_vpn_service_activate (NMVPNService *service,
-                                  NMVPNConnection *vpn,
+gboolean nm_vpn_service_activate (NMVpnService *service,
+                                  NMVpnConnection *vpn,
                                   GError **error);
 
-void nm_vpn_service_stop_connections (NMVPNService *service,
+void nm_vpn_service_stop_connections (NMVpnService *service,
                                       gboolean quitting,
-                                      NMVPNConnectionStateReason reason);
+                                      NMVpnConnectionStateReason reason);
 
 #endif  /* NM_VPN_VPN_SERVICE_H */
